@@ -154,7 +154,7 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 #define QUIRK_VIBALL	(1 << 20)	/* Vibrato in all frames */
 #define QUIRK_VIBINV	(1 << 21)	/* Vibrato has inverse waveform */
 #define QUIRK_PRENV	(1 << 22)	/* Portamento resets envelope & fade */
-/*#define QUIRK_S3MLFO	(1 << 23)*/	/* S3M-style LFO waveforms */
+#define QUIRK_ITOLDFX	(1 << 23)	/* IT old effects mode */
 #define QUIRK_S3MRTG	(1 << 24)	/* S3M-style retrig when count == 0 */
 #define QUIRK_RTDELAY	(1 << 25)	/* Delay effect retrigs instrument */
 #define QUIRK_FT2BUGS	(1 << 26)	/* FT2 bug compatibility */
@@ -187,6 +187,8 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 #define MED_TIME_FACTOR		2.64
 
 #define MAX_SEQUENCES		16
+#define MAX_SAMPLE_SIZE		0x10000000
+#define MAX_SAMPLES		1024
 
 struct ord_data {
 	int speed;
@@ -225,6 +227,7 @@ struct module_data {
 	int c4rate;			/* C4 replay rate */
 	int volbase;			/* Volume base */
 	int gvolbase;			/* Global volume base */
+	int gvol;			/* Global volume */
 	int *vol_table;			/* Volume translation table */
 	int quirk;			/* player quirks */
 #define READ_EVENT_MOD	0
