@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -82,27 +82,35 @@ static void set_position(struct context_data *ctx, int pos, int dir)
 		p->sequence = seq;
 
 		if (pos >= 0) {
-			if (mod->xxo[pos] == 0xff) {
-				return;
-			}
+			int pat;
 
-			while (mod->xxo[pos] == 0xfe && pos > start) {
+			while (mod->xxo[pos] == 0xfe) {
 				if (dir < 0) {
-					pos--;
+					if (pos > start) {
+						pos--;
+					}
 				} else {
 					pos++;
 				}
 			}
+			pat = mod->xxo[pos];
 
-			if (pos > p->scan[seq].ord) {
-				f->end_point = 0;
-			} else {
-				f->num_rows = mod->xxp[mod->xxo[p->ord]]->rows;
-				f->end_point = p->scan[seq].num;
+			if (pat < mod->pat) {
+				if (pat == 0xff) {
+					return;
+				}
+
+				if (pos > p->scan[seq].ord) {
+					f->end_point = 0;
+				} else {
+					f->num_rows = mod->xxp[pat]->rows;
+					f->end_point = p->scan[seq].num;
+					f->jumpline = 0;
+				}
 			}
 		}
 
-		if (pos < m->mod.len) {
+		if (pos < mod->len) {
 			if (pos == 0) {
 				p->pos = -1;
 			} else {
