@@ -90,9 +90,14 @@ static char *oggdec(FILE *f, int len, int res, int *newlen)
 	int i, n, ch;
 	/*int size;*/
 	uint8 *data, *pcm;
-	int16 *pcm16;
+	int16 *pcm16 = NULL;
 	uint32 id;
 
+	/* Sanity check */
+	if (len < 4) {
+		return NULL;
+	}
+
 	/*size =*/ read32l(f);
 	id = read32b(f);
 	fseek(f, -8, SEEK_CUR);
@@ -111,8 +116,10 @@ static char *oggdec(FILE *f, int len, int res, int *newlen)
 	n = stb_vorbis_decode_memory(data, len, &ch, &pcm16);
 	free(data);
 
-	if (n <= 0)
+	if (n <= 0) {
+		free(pcm16);
 		return NULL;
+	}
 
 	pcm = (uint8 *)pcm16;
 
@@ -189,6 +196,8 @@ static int decrunch_oxm(FILE *f, FILE *fo)
 		/* Read sample headers */
 		for (j = 0; j < nsmp; j++) {
 			xi[j].len = read32l(f);
+			if (xi[j].len > MAX_SAMPLE_SIZE)
+				return -1;
 			fread(xi[j].buf, 1, 36, f);
 		}
 
