@@ -74,7 +74,7 @@ static void do_toneporta(struct module_data *m,
 	if (note >= 1 && note <= 0x80 && (uint32)xc->ins < m->mod.ins) {
 		note--;
 		xc->porta.target = note_to_period(note + sub->xpo +
-			instrument->map[xc->key].xpo, xc->finetune,
+			instrument->map[xc->key_porta].xpo, xc->finetune,
 			HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 	}
 	xc->porta.dir = xc->period < xc->porta.target ? 1 : -1;
@@ -91,6 +91,11 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	uint8 note, fxp, fxt;
 	int h, l;
 
+	/* key_porta is IT only */
+	if (m->read_event_type != READ_EVENT_IT) {
+		xc->key_porta = xc->key;
+	}
+
 	note = e->note;
 	if (fnum == 0) {
 		fxt = e->fxt;
@@ -252,13 +257,16 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		    || e->fxt != FX_EXTENDED		/* or not delay */
 		    || MSN(e->fxp) != EX_DELAY) {
 			xc->pan.val = fxp;
+			xc->pan.surround = 0;
 		}
 		break;
 	case FX_OFFSET:		/* Set sample offset */
 		EFFECT_MEMORY(fxp, xc->offset.memory);
 		SET(OFFSET);
 		if (note) {
-			xc->offset.val = xc->offset.val2 = fxp << 8;
+			xc->offset.val &= xc->offset.val & ~0xffff;
+			xc->offset.val |= fxp << 8;
+			xc->offset.val2 = fxp << 8;
 		}
 		if (e->ins) {
 			xc->offset.val2 = fxp << 8;
@@ -359,7 +367,11 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		    EFFECT_MEMORY(fxp, xc->fine_porta.down_memory);
 		    goto fx_f_porta_dn;
 		case EX_GLISS:		/* Glissando toggle */
-			xc->gliss = fxp;
+			if (fxp) {
+				SET(GLISSANDO);
+			} else {
+				RESET(GLISSANDO);
+			}
 			break;
 		case EX_VIBRATO_WF:	/* Set vibrato waveform */
 			fxp &= 3;
@@ -451,6 +463,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		SET(FINE_VOLS);
 		xc->vol.fslide = -fxp;
 		break;
+
 	case FX_F_PORTA_UP:	/* Fine portamento up */
 	    fx_f_porta_up:
 		if (fxp) {
@@ -512,6 +525,33 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			f->rowdelay_set = 1;
 		}
 		break;
+
+	/* From the OpenMPT VolColMemory.it test case:
+	 * "Volume column commands a, b, c and d (volume slide) share one
+	 *  effect memory, but it should not be shared with Dxy in the effect
+	 *  column. 
+	 */
+	case FX_VSLIDE_UP_2:	/* Fine volume slide up */
+		EFFECT_MEMORY(fxp, xc->vol.memory2);
+		SET(VOL_SLIDE_2);
+		xc->vol.slide2 = fxp;
+		break;
+	case FX_VSLIDE_DN_2:	/* Fine volume slide down */
+		EFFECT_MEMORY(fxp, xc->vol.memory2);
+		SET(VOL_SLIDE_2);
+		xc->vol.slide2 = -fxp;
+		break;
+	case FX_F_VSLIDE_UP_2:	/* Fine volume slide up */
+		EFFECT_MEMORY(fxp, xc->vol.memory2);
+		SET(FINE_VOLS_2);
+		xc->vol.fslide2 = fxp;
+		break;
+	case FX_F_VSLIDE_DN_2:	/* Fine volume slide down */
+		EFFECT_MEMORY(fxp, xc->vol.memory2);
+		SET(FINE_VOLS_2);
+		xc->vol.fslide2 = -fxp;
+		break;
+
 #endif
 
 	case FX_GLOBALVOL:	/* Set global volume */
@@ -726,7 +766,9 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		}
 		break;
 	case FX_FLT_CUTOFF:
-		xc->filter.cutoff = fxp;
+		if (fxp < 0xfe || xc->filter.resonance > 0) {
+			xc->filter.cutoff = fxp;
+		}
 		break;
 	case FX_FLT_RESN:
 		xc->filter.resonance = fxp;
@@ -738,6 +780,13 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	case FX_PANBRELLO_WF:	/* Panbrello waveform */
 		set_lfo_waveform(&xc->panbrello.lfo, fxp & 3);
 		break;
+	case FX_HIOFFSET:	/* High offset */
+		xc->offset.val &= 0xffff;
+		xc->offset.val |= fxp << 16;
+		break;
+	case FX_SURROUND:
+		xc->pan.surround = fxp;
+		break;
 #endif
 
 #ifndef LIBXMP_CORE_PLAYER
