@@ -69,11 +69,6 @@ int get_envelope(struct xmp_envelope *env, int x, int def)
 	/* interpolate */
 	y1 = data[index + 1];
 	x2 = data[index + 2];
-
-	if (env->flg & XMP_ENVELOPE_LOOP && index == (env->lpe << 1)) {
-		index = (env->lps - 1) * 2;
-	}
-
 	y2 = data[index + 3];
 
 	return x2 == x1 ? y2 : ((y2 - y1) * (x - x1) / (x2 - x1)) + y1;
@@ -83,14 +78,7 @@ static int update_envelope_xm(struct xmp_envelope *env, int x, int release)
 {
 	int16 *data = env->data;
 	int has_loop, has_sus;
-	int lpe, lps, sus, sue;
-
-	if (x < 0)
-		return -1;
-
-	if (~env->flg & XMP_ENVELOPE_ON || env->npt <= 0) {
-		return x;
-	}
+	int lpe, lps, sus;
 
 	has_loop = env->flg & XMP_ENVELOPE_LOOP;
 	has_sus = env->flg & XMP_ENVELOPE_SUS;
@@ -98,7 +86,6 @@ static int update_envelope_xm(struct xmp_envelope *env, int x, int release)
 	lps = env->lps << 1;
 	lpe = env->lpe << 1;
 	sus = env->sus << 1;
-	sue = env->sue << 1;
 
 	/* FT2 and IT envelopes behave in a different way regarding loops,
 	 * sustain and release. When the sustain point is at the end of the
@@ -111,16 +98,14 @@ static int update_envelope_xm(struct xmp_envelope *env, int x, int release)
 			has_sus = 0;
 	}
 
-	/* FT2 doesn't have sustain loop */
-	if (!release && has_sus && x == data[sus]) {
-		/* stay in the sustain point */
-		x--;
-	}
-
-	if (x < 0xffff)	{	/* increment tick */
-		x++;
+	/* If enabled, stay at the sustain point */
+	if (has_sus && !release) {
+		if (x >= data[sus]) {
+			x = data[sus];
+		}
 	}
 
+	/* Envelope loops */
 	if (has_loop && x >= data[lpe]) {
 		if (!(release && has_sus && sus == lpe))
 			x = data[lps];
@@ -131,19 +116,12 @@ static int update_envelope_xm(struct xmp_envelope *env, int x, int release)
 
 #ifndef LIBXMP_CORE_DISABLE_IT
 
-static int update_envelope_it(struct xmp_envelope *env, int x, int release)
+static int update_envelope_it(struct xmp_envelope *env, int x, int release, int key_off)
 {
 	int16 *data = env->data;
 	int has_loop, has_sus;
 	int lpe, lps, sus, sue;
 
-	if (x < 0)
-		return -1;
-
-	if (~env->flg & XMP_ENVELOPE_ON || env->npt <= 0) {
-		return x;
-	}
-
 	has_loop = env->flg & XMP_ENVELOPE_LOOP;
 	has_sus = env->flg & XMP_ENVELOPE_SUS;
 
@@ -152,28 +130,21 @@ static int update_envelope_it(struct xmp_envelope *env, int x, int release)
 	sus = env->sus << 1;
 	sue = env->sue << 1;
 
-	if (env->flg & XMP_ENVELOPE_SLOOP) {
-		if (!release && has_sus) {
-			if (x == data[sue])
-				x = data[sus] - 1;
-		} else if (has_loop) {
-			if (x == data[lpe])
-				x = data[lps] - 1;
+	/* Release at the end of a sustain loop, run another loop */
+	if (has_sus && key_off && x == data[sue] + 1) {
+		x = data[sus];
+	} else
+	/* If enabled, stay in the sustain loop */
+	if (has_sus && !release) {
+		if (x == data[sue] + 1) {
+			x = data[sus];
 		}
-	} else {
-		if (!release && has_sus && x == data[sus]) {
-			/* stay in the sustain point */
-			x--;
+	} else
+	/* Finally, execute the envelope loop */
+	if (has_loop) {
+		if (x > data[lpe]) {
+			x = data[lps];
 		}
-
-		if (has_loop && x == data[lpe]) {
-	    		if (!(release && has_sus && sus == lpe))
-				x = data[lps] - 1;
-		}
-	}
-
-	if (x < 0xffff)	{	/* increment tick */
-		x++;
 	}
 
 	return x;
@@ -181,11 +152,23 @@ static int update_envelope_it(struct xmp_envelope *env, int x, int release)
 
 #endif
 
-int update_envelope(struct xmp_envelope *env, int x, int release, int it_env)
+int update_envelope(struct xmp_envelope *env, int x, int release, int key_off, int it_env)
 {
+	if (x < 0xffff)	{	/* increment tick */
+		x++;
+	}
+
+	if (x < 0) {
+		return -1;
+	}
+
+	if (~env->flg & XMP_ENVELOPE_ON || env->npt <= 0) {
+		return x;
+	}
+
 #ifndef LIBXMP_CORE_DISABLE_IT
 	return it_env ?
-		update_envelope_it(env, x, release) :
+		update_envelope_it(env, x, release, key_off) :
 		update_envelope_xm(env, x, release);
 #else
 	return update_envelope_xm(env, x, release);
