@@ -342,7 +342,7 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 	struct context_data *ctx = (struct context_data *)opaque;
 	struct module_data *m = &ctx->m;
 	struct xmp_module *mod = &m->mod;
-	int i, ret;
+	int i, j, ret;
 	int test_result, load_result;
 
 	load_prologue(ctx);
@@ -351,6 +351,7 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 	test_result = load_result = -1;
 	for (i = 0; format_loader[i] != NULL; i++) {
 		hio_seek(h, 0, SEEK_SET);
+		D_(D_WARN "test %s", format_loader[i]->name);
 		test_result = format_loader[i]->test(h, NULL, 0);
 		if (test_result == 0) {
 			hio_seek(h, 0, SEEK_SET);
@@ -372,28 +373,48 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 	}
 
 	if (load_result < 0) {
-		xmp_release_module(opaque);
-		return -XMP_ERROR_LOAD;
+		goto err_load;
 	}
 
 	/* Sanity check */
 	if (mod->chn > XMP_MAX_CHANNELS || mod->len > XMP_MAX_MOD_LENGTH) {
-		xmp_release_module(opaque);
-		return -XMP_ERROR_LOAD;
+		goto err_load;
+	}
+
+	/* Sanity check */
+	if (mod->xxp == NULL) {
+		goto err_load;
+	}
+	for (i = 0; i < mod->pat; i++) {
+		if (mod->xxp[i] == NULL) {
+			goto err_load;
+		}
+		for (j = 0; j < mod->chn; j++) {
+			int t = mod->xxp[i]->index[j];
+			if (t < 0 || t >= mod->trk || mod->xxt[t] == NULL) {
+				goto err_load;
+			}
+		}
 	}
 
 	adjust_string(mod->name);
 	load_epilogue(ctx);
 
 	ret = prepare_scan(ctx);
-	if (ret < 0)
+	if (ret < 0) {
+		xmp_release_module(opaque);
 		return ret;
+	}
 
 	scan_sequences(ctx);
 
 	ctx->state = XMP_STATE_LOADED;
 
 	return 0;
+
+    err_load:
+	xmp_release_module(opaque);
+	return -XMP_ERROR_LOAD;
 }
 
 int xmp_load_module(xmp_context opaque, char *path)
@@ -572,8 +593,9 @@ void xmp_release_module(xmp_context opaque)
 
 	if (mod->xxs != NULL) {
 		for (i = 0; i < mod->smp; i++) {
-			if (mod->xxs[i].data != NULL)
+			if (mod->xxs[i].data != NULL) {
 				free(mod->xxs[i].data - 4);
+			}
 		}
 		free(mod->xxs);
 	}
