@@ -144,7 +144,7 @@ static int read_abk_song(HIO_HANDLE *f, struct abk_song *song, uint32 songs_sect
     int i;
     uint32 song_section;
 
-    /* move to the start of the songs data sectio */
+    /* move to the start of the songs data section */
     hio_seek(f, songs_section_offset, SEEK_SET);
 
     if (hio_read16b(f) != 1)
@@ -180,7 +180,7 @@ static int read_abk_song(HIO_HANDLE *f, struct abk_song *song, uint32 songs_sect
  * @param pattern_offset_abs the absolute file offset to the start of the patter to read.
  * @return returns the size of the pattern.
  */
-static uint16 read_abk_pattern(HIO_HANDLE *f, struct xmp_event *events, uint32 pattern_offset_abs)
+static int read_abk_pattern(HIO_HANDLE *f, struct xmp_event *events, uint32 pattern_offset_abs)
 {
     uint8 position;
     uint8 command;
@@ -307,9 +307,13 @@ static uint16 read_abk_pattern(HIO_HANDLE *f, struct xmp_event *events, uint32 p
                 jumped = 1;
                 break;
             default:
+#if 0
                 /* write out an error for any unprocessed commands.*/
                 D_(D_WARN "ABK UNPROCESSED COMMAND: %x,%x\n", command, param);
                 break;
+#else
+		return -1;
+#endif
             }
         }
         else
@@ -447,6 +451,13 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
     main_header.songs_offset = hio_read32b(f);
     main_header.patterns_offset = hio_read32b(f);
 
+    /* Sanity check */
+    if (main_header.instruments_offset > 0x00100000 ||
+        main_header.songs_offset > 0x00100000 ||
+        main_header.patterns_offset > 0x00100000) {
+            return -1;
+    }
+
     inst_section_size = main_header.instruments_offset;
     D_(D_INFO "Sample Bytes: %d", inst_section_size);
 
@@ -474,6 +485,11 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
     mod->ins = hio_read16b(f);
     mod->smp = mod->ins;
 
+    /* Sanity check */
+    if (mod->pat > 256 || mod->ins > 255) {
+	return -1;
+    }
+
     /* Read and convert instruments and samples */
 
     if (instrument_init(mod) < 0)
@@ -559,23 +575,30 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
             return -1;
         }
 
-        for (k = 0; k  < mod->chn; k++)
+        for (k = 0; k < mod->chn; k++)
         {
             pattern = hio_read16b(f);
-            read_abk_pattern(f,  mod->xxt[(i*mod->chn)+k]->event, AMOS_MAIN_HEADER + main_header.patterns_offset + pattern);
+            if (read_abk_pattern(f,  mod->xxt[(i*mod->chn)+k]->event, AMOS_MAIN_HEADER + main_header.patterns_offset + pattern) < 0) {
+    		free(playlist.pattern);
+		return -1;
+	    }
         }
 
         i++;
     }
 
-    /* now push all the patterns into the module and set the length */
-    i = 0;
+    /* Sanity check */
+    if (playlist.length > 256) {
+    	free(playlist.pattern);
+	return -1;
+    }
 	
-    for (j=0; j< playlist.length; j++)
+    mod->len = playlist.length;
+
+    /* now push all the patterns into the module and set the length */
+    for (i = 0; i < playlist.length; i++)
     {
-        /* increment the length */
-        mod->len++;
-        mod->xxo[i++] = playlist.pattern[j];
+        mod->xxo[i] = playlist.pattern[i];
     }
 
     /* free up some memory here */
