@@ -167,9 +167,20 @@ static int load_packed_patterns(struct module_data *m, HIO_HANDLE *f)
 	}
 	mod->trk++;
 
-	stored_tracks = hio_read16l(f);
+	/* Sanity check */
+	if (mod->trk > mod->pat * 9) {
+		return -1;
+	}
 
-	D_(D_INFO "Stored tracks: %d", w);
+	stored_tracks = hio_read16l(f);
+	
+	/* Sanity check */
+	if (stored_tracks <= 0) {
+		return -1;
+	}
+
+	D_(D_INFO "Tracks: %d", mod->trk);
+	D_(D_INFO "Stored tracks: %d", stored_tracks);
 
 	mod->xxt = calloc(sizeof(struct xmp_track *), mod->trk);
 	if (mod->xxt == NULL)
@@ -177,8 +188,15 @@ static int load_packed_patterns(struct module_data *m, HIO_HANDLE *f)
 
 	for (i = 0; i < stored_tracks; i++) {
 		w = hio_read16l(f);
-		if (track_alloc(mod, w, 64) < 0)
+
+		/* Sanity check */
+		if (w < 0 || w >= mod->trk || mod->xxt[w] != NULL) {
 			return -1;
+		}
+
+		if (track_alloc(mod, w, 64) < 0) {
+			return -1;
+		}
 
 		for (r = 0; r < 64; r++) {
 			event = &mod->xxt[w]->event[r];
