@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -122,7 +122,7 @@ static int amf_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	D_(D_INFO "Stored patterns: %d", mod->pat);
 
-	mod->xxp = calloc(sizeof(struct xmp_pattern *), mod->pat + 1);
+	mod->xxp = calloc(sizeof(struct xmp_pattern *), mod->pat);
 	if (mod->xxp == NULL)
 		return -1;
 
@@ -132,6 +132,9 @@ static int amf_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		mod->xxp[i]->rows = ver >= 0x0e ? hio_read16l(f) : 64;
 
+		if (mod->xxp[i]->rows > 256)
+			return -1;
+
 		for (j = 0; j < mod->chn; j++) {
 			uint16 t = hio_read16l(f);
 			mod->xxp[i]->index[j] = t;
@@ -293,7 +296,7 @@ static int amf_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		uint8 t1, t2, t3;
 		int size;
 
-		if (track_alloc(mod, i, 64) < 0)
+		if (track_alloc(mod, i, 64) < 0)	/* FIXME! */
 			return -1;
 
 		size = hio_read24l(f);
@@ -308,6 +311,10 @@ static int amf_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			if (t1 == 0xff && t2 == 0xff && t3 == 0xff)
 				break;
 
+			/* Sanity check */
+			if (t1 >= mod->xxt[i]->rows)
+				return -1;
+
 			event = &mod->xxt[i]->event[t1];
 
 			if (t2 < 0x7f) {		/* note */
@@ -472,7 +479,6 @@ static int amf_load(struct module_data *m, HIO_HANDLE *f, const int start)
 				event->fxt = fxt;
 				event->fxp = fxp;
 			}
-
 		}
 	}
 
