@@ -25,6 +25,12 @@
 
 #define MAGIC_MUSX	MAGIC4('M','U','S','X')
 #define MAGIC_MNAM	MAGIC4('M','N','A','M')
+#define MAGIC_SNAM	MAGIC4('S','N','A','M')
+#define MAGIC_SVOL	MAGIC4('S','V','O','L')
+#define MAGIC_SLEN	MAGIC4('S','L','E','N')
+#define MAGIC_ROFS	MAGIC4('R','O','F','S')
+#define MAGIC_RLEN	MAGIC4('R','L','E','N')
+#define MAGIC_SDAT	MAGIC4('S','D','A','T')
 
 
 static int arch_test (HIO_HANDLE *, char *, const int);
@@ -64,8 +70,9 @@ static uint8 convert_vol(uint8 vol) {
 
 static int arch_test(HIO_HANDLE *f, char *t, const int start)
 {
-	if (hio_read32b(f) != MAGIC_MUSX)
+	if (hio_read32b(f) != MAGIC_MUSX) {
 		return -1;
+	}
 
 	hio_read32l(f);
 
@@ -73,6 +80,11 @@ static int arch_test(HIO_HANDLE *f, char *t, const int start)
 		uint32 id = hio_read32b(f);
 		uint32 len = hio_read32l(f);
 
+		/* Sanity check */
+		if (len > 0x100000) {
+			return -1;
+		}
+
 		if (id == MAGIC_MNAM) {
 			read_title(f, t, 32);
 			return 0;
@@ -186,6 +198,11 @@ static int get_mvox(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 
 	mod->chn = hio_read32l(f);
 
+	/* Sanity check */
+	if (mod->chn < 1 || mod->chn > 8) {
+		return -1;
+	}
+
 	return 0;
 }
 
@@ -195,9 +212,11 @@ static int get_ster(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	struct local_data *data = (struct local_data *)parm;
 	int i;
 
-	hio_read(data->ster, 1, 8, f);
+	if (hio_read(data->ster, 1, 8, f) != 8) {
+		return -1;
+	}
 	
-	for (i=0; i < mod->chn; i++) {
+	for (i = 0; i < mod->chn; i++) {
 		if (data->ster[i] > 0 && data->ster[i] < 8) {
 			mod->xxc[i].pan = 42 * data->ster[i] - 40;
 		}
@@ -210,7 +229,8 @@ static int get_mnam(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
 
-	hio_read(mod->name, 1, 32, f);
+	if (hio_read(mod->name, 1, 32, f) != 32)
+		return -1;
 
 	return 0;
 }
@@ -228,6 +248,10 @@ static int get_mlen(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 
 	mod->len = hio_read32l(f);
 
+	/* Sanity check */
+	if (mod->len > 0xff)
+		return -1;
+
 	return 0;
 }
 
@@ -237,6 +261,10 @@ static int get_pnum(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 
 	mod->pat = hio_read32l(f);
 
+	/* Sanity check */
+	if (mod->pat > 64 )
+		return -1;
+
 	return 0;
 }
 
@@ -244,7 +272,8 @@ static int get_plen(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct local_data *data = (struct local_data *)parm;
 
-	hio_read(data->rows, 1, 64, f);
+	if (hio_read(data->rows, 1, 64, f) != 64)
+		return -1;
 
 	return 0;
 }
@@ -277,6 +306,10 @@ static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 			return -1;
 	}
 
+	/* Sanity check */
+	if (data->max_pat >= mod->pat || data->max_pat >= 64)
+		return -1;
+
         i = data->max_pat;
 
 	if (pattern_tracks_alloc(mod, i, data->rows[i]) < 0)
@@ -333,28 +366,43 @@ static int get_samp(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	if (subinstrument_alloc(mod, i, 1) < 0)
 		return -1;
 
-	hio_read32l(f);	/* SNAM */
+	if (hio_read32b(f) != MAGIC_SNAM)	/* SNAM */
+		return -1;
+
 	{
 		/* should usually be 0x14 but zero is not unknown */
 		int name_len = hio_read32l(f);
-		if (name_len < 32)
-			hio_read(mod->xxi[i].name, 1, name_len, f);
+
+		/* Sanity check */
+		if (name_len < 0 || name_len > 32)
+			return -1;
+
+		hio_read(mod->xxi[i].name, 1, name_len, f);
 	}
-	hio_read32l(f);	/* SVOL */
+
+	if (hio_read32b(f) != MAGIC_SVOL)	/* SVOL */
+		return -1;
 	hio_read32l(f);
 	/* mod->xxi[i].sub[0].vol = convert_vol(hio_read32l(f)); */
 	mod->xxi[i].sub[0].vol = hio_read32l(f) & 0xff;
-	hio_read32l(f);	/* SLEN */
+
+	if (hio_read32b(f) != MAGIC_SLEN)	/* SLEN */
+		return -1;
 	hio_read32l(f);
 	mod->xxs[i].len = hio_read32l(f);
-	hio_read32l(f);	/* ROFS */
+
+	if (hio_read32b(f) != MAGIC_ROFS)	/* ROFS */
+		return -1;
 	hio_read32l(f);
 	mod->xxs[i].lps = hio_read32l(f);
-	hio_read32l(f);	/* RLEN */
+
+	if (hio_read32b(f) != MAGIC_RLEN)	/* RLEN */
+		return -1;
 	hio_read32l(f);
 	mod->xxs[i].lpe = hio_read32l(f);
 
-	hio_read32l(f);	/* SDAT */
+	if (hio_read32b(f) != MAGIC_SDAT)	/* SDAT */
+		return -1;
 	hio_read32l(f);
 	hio_read32l(f);	/* 0x00000000 */
 
