@@ -76,6 +76,10 @@ int pattern_init(struct xmp_module *mod)
 
 int pattern_alloc(struct xmp_module *mod, int num)
 {
+	/* Sanity check */
+	if (num < 0 || num >= mod->pat || mod->xxp[num] != NULL)
+		return -1;
+
 	mod->xxp[num] = calloc(1, sizeof (struct xmp_pattern) +
         				sizeof (int) * (mod->chn - 1));
 	if (mod->xxp[num] == NULL)
@@ -86,8 +90,12 @@ int pattern_alloc(struct xmp_module *mod, int num)
 
 int track_alloc(struct xmp_module *mod, int num, int rows)
 {
-	mod->xxt[num] = calloc (sizeof (struct xmp_track) +
-				sizeof (struct xmp_event) * (rows - 1), 1);
+	/* Sanity check */
+	if (num >= mod->trk || mod->xxt[num] != NULL)
+		return -1;
+
+	mod->xxt[num] = calloc(sizeof (struct xmp_track) +
+			       sizeof (struct xmp_event) * (rows - 1), 1);
 	if (mod->xxt[num] == NULL)
 		return -1;
 
@@ -100,6 +108,7 @@ int tracks_in_pattern_alloc(struct xmp_module *mod, int num)
 {
 	int i;
 
+	D_(D_INFO "Alloc %d tracks of %d rows", mod->chn, mod->xxp[num]->rows);
 	for (i = 0; i < mod->chn; i++) {
 		int t = num * mod->chn + i;
 		int rows = mod->xxp[num]->rows;
@@ -115,8 +124,13 @@ int tracks_in_pattern_alloc(struct xmp_module *mod, int num)
 
 int pattern_tracks_alloc(struct xmp_module *mod, int num, int rows)
 {
+	/* Sanity check */
+	if (rows < 0 || rows > 256)
+		return -1;
+
 	if (pattern_alloc(mod, num) < 0)
 		return -1;
+
 	mod->xxp[num]->rows = rows;
 
 	if (tracks_in_pattern_alloc(mod, num) < 0)
