@@ -67,6 +67,10 @@ static int get_info(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	mod->pat = hio_read16b(f);
 	mod->chn = hio_read16b(f);
 
+	/* Sanity check */
+	if (mod->ins > 255 || mod->pat > 256 || mod->chn > XMP_MAX_CHANNELS)
+		return -1;
+
 	mod->trk = mod->pat * mod->chn;
 
 	if (instrument_init(mod) < 0)
@@ -93,6 +97,11 @@ static int get_song(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	mod->len = hio_read16b(f);
 	D_(D_INFO "Song length: %d patterns", mod->len);
 
+	/* Sanity check */
+	if (mod->len > 256) {
+		return -1;
+	}
+
 	for (i = 0; i < mod->len; i++)
 		mod->xxo[i] = hio_read16b(f);
 
