@@ -93,6 +93,12 @@ static int get_s_q_(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 
 	mod->len = hio_read16b(f);
 	mod->rst = hio_read16b(f);
+
+	/* Sanity check */
+	if (mod->len > 256 || mod->rst > 255) {
+		return -1;
+	}
+
 	hio_read32b(f);	/* reserved */
 
 	for (maxpat = i = 0; i < 128; i++) {
@@ -205,6 +211,14 @@ static int get_dapt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	pat = hio_read16b(f);
 	rows = hio_read16b(f);
 
+	/* Sanity check */
+	if (pat < 0 || pat >= mod->pat || rows < 0 || rows > 256) {
+		return -1;
+	}
+	if (pat < data->last_pat) {
+		return -1;
+	}
+
 	for (i = data->last_pat; i <= pat; i++) {
 		if (pattern_tracks_alloc(mod, i, rows) < 0)
 			return -1;
@@ -246,6 +260,11 @@ static int get_dait(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	}
 
 	if (size > 2) {
+		/* Sanity check */
+		if (data->insnum >= mod->ins) {
+			return -1;
+		}
+
 		int ret = load_sample(m, f, SAMPLE_FLAG_BIGEND,
 			&mod->xxs[mod->xxi[data->insnum].sub[0].sid], NULL);
 
@@ -262,11 +281,12 @@ static int dt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 {
 	iff_handle handle;
 	struct local_data data;
+	struct xmp_module *mod = &m->mod;
 	int ret, i;
 
 	LOAD_INIT();
 
-	data.pflag = data.sflag = 0;
+	memset(&data, 0, sizeof (struct local_data));
 	
 	handle = iff_new();
 	if (handle == NULL)
@@ -289,10 +309,13 @@ static int dt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	if (ret < 0)
 		return -1;
 
-	//fix rest patterns
-	for (i = data.last_pat; i < m->mod.pat; i++) {
-		if (pattern_tracks_alloc(&m->mod, i, 64) < 0)
-			return -1;
+	/* alloc remaining patterns */
+	if (mod->xxp != NULL) {
+		for (i = data.last_pat; i < mod->pat; i++) {
+			if (pattern_tracks_alloc(mod, i, 64) < 0) {
+				return -1;
+			}
+		}
 	}
 
 	return 0;
