@@ -305,12 +305,12 @@ static int get_pbod(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 						note = (fxt>>4)*12 +
 							(fxt & 0x0f) + 1;
 						event->note = note;
-abort();
 						fxt = FX_TONEPORTA;
 						fxp = (fxp + 1) * 2;
 						break; }
 					default:
 D_(D_CRIT "p%d r%d c%d: compressed event %02x %02x\n", i, r, chan, fxt, fxp);
+						return -1;
 					}
 				} else
 				switch (fxt) {
