@@ -81,6 +81,11 @@ static int mgt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	hio_read16b(f);			/* reserved */
 	hio_read32b(f);			/* reserved */
 
+	/* Sanity check */
+	if (mod->ins > 64) {
+		return -1;
+	}
+
 	sng_ptr = hio_read32b(f);
 	seq_ptr = hio_read32b(f);
 	ins_ptr = hio_read32b(f);
@@ -102,6 +107,11 @@ static int mgt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	hio_read8(f);			/* master L */
 	hio_read8(f);			/* master R */
 
+	/* Sanity check */
+	if (mod->len > 256 || mod->rst > 255) {
+		return -1;
+	}
+
 	for (i = 0; i < mod->chn; i++) {
 		hio_read16b(f);		/* pan */
 	}
@@ -111,9 +121,15 @@ static int mgt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	/* Sequence */
 
 	hio_seek(f, start + seq_ptr, SEEK_SET);
-	for (i = 0; i < mod->len; i++)
+	for (i = 0; i < mod->len; i++) {
 		mod->xxo[i] = hio_read16b(f);
 
+		/* Sanity check */
+		if (mod->xxo[i] >= mod->pat) {
+			return -1;
+		}
+	}
+
 	/* Instruments */
 
 	if (instrument_init(mod) < 0)
@@ -130,6 +146,12 @@ static int mgt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		hio_read(mod->xxi[i].name, 1, 32, f);
 		sdata[i] = hio_read32b(f);
 		mod->xxs[i].len = hio_read32b(f);
+
+		/* Sanity check */
+		if (mod->xxs[i].len > MAX_SAMPLE_SIZE) {
+			return -1;
+		}
+
 		mod->xxs[i].lps = hio_read32b(f);
 		mod->xxs[i].lpe = mod->xxs[i].lps + hio_read32b(f);
 		hio_read32b(f);
@@ -181,6 +203,10 @@ static int mgt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		rows = hio_read16b(f);
 
+		/* Sanity check */
+		if (rows > 255)
+			return -1;
+
 		if (track_alloc(mod, i, rows) < 0)
 			return -1;
 
@@ -191,6 +217,10 @@ static int mgt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			b = hio_read8(f);
 			j += b & 0x03;
 
+			/* Sanity check */
+			if (j >= rows)
+				return -1;
+
 			note = 0;
 			event = &mod->xxt[i]->event[j];
 			if (b & 0x04)
@@ -290,9 +320,11 @@ static int mgt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	}
 
 	/* Extra track */
-	mod->xxt[0] = calloc(sizeof(struct xmp_track) +
+	if (mod->trk > 0) {
+		mod->xxt[0] = calloc(sizeof(struct xmp_track) +
 			sizeof(struct xmp_event) * 64 - 1, 1);
-	mod->xxt[0]->rows = 64;
+		mod->xxt[0]->rows = 64;
+	}
 
 	/* Read and convert patterns */
 	D_(D_INFO "Stored patterns: %d", mod->pat);
@@ -300,12 +332,29 @@ static int mgt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	hio_seek(f, start + pat_ptr, SEEK_SET);
 
 	for (i = 0; i < mod->pat; i++) {
+		int rows;
+
 		if (pattern_alloc(mod, i) < 0)
 			return -1;
 
-		mod->xxp[i]->rows = hio_read16b(f);
+		rows = hio_read16b(f);
+
+		/* Sanity check */
+		if (rows > 256) {
+			return -1;
+		}
+
+		mod->xxp[i]->rows = rows;
+
 		for (j = 0; j < mod->chn; j++) {
-			mod->xxp[i]->index[j] = hio_read16b(f) - 1;
+			int track = hio_read16b(f) - 1;
+
+			/* Sanity check */
+			if (track >= mod->trk) {
+				return -1;
+			}
+
+			mod->xxp[i]->index[j] = track;
 		}
 	}
 
