@@ -174,8 +174,15 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	hio_read32b(f);
 	hio_read32b(f);
 	mod->len = hio_read16b(f);
-	for (i = 0; i < mod->len; i++)
+
+	/* Sanity check */
+	if (mod->len > 255) {
+		return -1;
+	}
+
+	for (i = 0; i < mod->len; i++) {
 		mod->xxo[i] = hio_read16b(f);
+	}
 
 	/*
 	 * convert header
@@ -243,6 +250,15 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		D_(D_INFO "iinfo_offset = 0x%08x", iinfo_offset);
 		expdata.i_ext_entries = hio_read16b(f);
 		expdata.i_ext_entrsz = hio_read16b(f);
+
+		/* Sanity check */
+		if (expsmp_offset < 0 ||
+		    expdata.s_ext_entries < 0 ||
+		    expdata.s_ext_entrsz < 0 ||
+		    iinfo_offset < 0) {
+			return -1;
+		}
+
 		hio_read32b(f);
 		hio_read32b(f);
 		hio_read32b(f);
@@ -288,6 +304,10 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			mod->chn = block.numtracks;
 	}
 
+	/* Sanity check */
+	if (mod->chn <= 0 || mod->chn > XMP_MAX_CHANNELS)
+		return -1;
+
 	mod->trk = mod->pat * mod->chn;
 
 	if (ver == 2)
@@ -398,7 +418,8 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			D_(D_INFO "[%2x] %-40.40s %d", i, mod->xxi[i].name, instr.type);
 		}
 
-		exp_smp.finetune = 0;
+		memset(&exp_smp, 0, sizeof(struct InstrExt));
+
 		if (expdata_offset && i < expdata.s_ext_entries) {
 			hio_seek(f, expsmp_offset + i * expdata.s_ext_entrsz,
 							SEEK_SET);
@@ -428,9 +449,7 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 				return -1;
 
 			continue;
-		}
-
-		if (instr.type == -1) {			/* Synthetic */
+		} else if (instr.type == -1) {		/* Synthetic */
 			int ret = mmd_load_synth_instrument(f, m, i, smp_idx,
 				&synth, &exp_smp, &song.sample[i]);
 
@@ -446,9 +465,7 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 				return -1;
 
 			continue;
-		}
-
-		if (instr.type >= 1 && instr.type <= 6) {	/* IFFOCT */
+		} else if (instr.type >= 1 && instr.type <= 6) { /* IFFOCT */
 			int ret;
 			const int oct = num_oct[instr.type - 1];
 
@@ -463,13 +480,11 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			smp_idx += oct;
 
 			continue;
-		}
-
-		/* Filter out stereo samples */
-		if ((instr.type & ~(S_16 | STEREO)) != 0)
+		} /* else if ((instr.type & ~(S_16 | STEREO)) != 0) {
+			D_(D_WARN "stereo sample unsupported");
+			mod->xxi[i].nsm = 0;
 			continue;
-
-		if (instr.type == 0) {			/* Sample */
+		} */ else if ((instr.type & ~S_16) == 0) {	/* Sample */
 			int ret;
 
 			hio_seek(f, start + smpl_offset + 6, SEEK_SET);
@@ -484,6 +499,10 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			smp_idx++;
 
 			continue;
+		} else {
+			/* Invalid instrument type */
+			D_(D_CRIT "invalid instrument type");
+			return -1;
 		}
 	}
 
