@@ -103,14 +103,22 @@ static const int fx[] = {
 static int get_cmod(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
-	int i, j, k;
+	int i;
 
 	mod->chn = 0;
 	for (i = 0; i < 4; i++) {
-		j = hio_read16b(f);
-		for (k = ! !j; k >= 0; k--) {
-			mod->chn++;
+		int pan = (((i + 1) / 2) % 2) * 0xff;
+		int p = 0x80 + (pan - 0x80) * m->defpan / 100;
+
+		if (hio_read16b(f) == 0) {
+			mod->xxc[mod->chn++].pan = p;
+		} else {
+			mod->xxc[mod->chn].flg |= XMP_CHANNEL_SPLIT | (i << 4);
+			mod->xxc[mod->chn++].pan = p;
+			mod->xxc[mod->chn].flg |= XMP_CHANNEL_SPLIT | (i << 4);
+			mod->xxc[mod->chn++].pan = p;
 		}
+
 	}
 
 	return 0;
@@ -123,6 +131,10 @@ static int get_samp(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	int i, j;
 	int looplen;
 
+	/* Sanity check */
+	if (size != 36 * 32)
+		return -1;
+
 	/* Should be always 36 */
 	mod->ins = size / 32;	/* sizeof(struct okt_instrument_header); */
 	mod->smp = mod->ins;
@@ -145,8 +157,8 @@ static int get_samp(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 
 		/* Sample size is always rounded down */
 		xxs->len = hio_read32b(f) & ~1;
-		xxs->lps = hio_read16b(f);
-		looplen = hio_read16b(f);
+		xxs->lps = hio_read16b(f) << 1;
+		looplen = hio_read16b(f) << 1;
 		xxs->lpe = xxs->lps + looplen;
 		xxs->flg = looplen > 2 ? XMP_SAMPLE_LOOP : 0;
 
@@ -192,6 +204,11 @@ static int get_plen(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	struct xmp_module *mod = &m->mod;
 
 	mod->len = hio_read16b(f);
+
+	/* Sanity check */
+	if (mod->len > 256)
+		return -1;
+
 	D_(D_INFO "Module length: %d", mod->len);
 
 	return 0;
@@ -201,7 +218,8 @@ static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
 
-	hio_read(mod->xxo, 1, mod->len, f);
+	if (hio_read(mod->xxo, 1, mod->len, f) != mod->len)
+		return -1;
 
 	return 0;
 }
