@@ -16,7 +16,7 @@
 #define MAGIC_0000	MAGIC4(0x0,0x0,0x0,0x0)
 
 
-static int depack_unic(FILE *in, FILE *out)
+static int depack_unic(HIO_HANDLE *in, FILE *out)
 {
 	uint8 c1, c2, c3, c4;
 	uint8 npat;
@@ -39,8 +39,8 @@ static int depack_unic(FILE *in, FILE *out)
 		write8(out, 0);
 
 		/* fine on ? */
-		c1 = read8(in);
-		c2 = read8(in);
+		c1 = hio_read8(in);
+		c2 = hio_read8(in);
 		j = (c1 << 8) + c2;
 		if (j != 0) {
 			if (j < 256)
@@ -52,15 +52,15 @@ static int depack_unic(FILE *in, FILE *out)
 		}
 
 		/* smp size */
-		len = read16b(in);
+		len = hio_read16b(in);
 		write16b(out, len);
 		ssize += len * 2;
 
-		read8(in);
+		hio_read8(in);
 		write8(out, fine);		/* fine */
-		write8(out, read8(in));		/* vol */
-		start = read16b(in);		/* loop start */
-		lsize = read16b(in);		/* loop size */
+		write8(out, hio_read8(in));		/* vol */
+		start = hio_read16b(in);		/* loop start */
+		lsize = hio_read16b(in);		/* loop size */
 
 		if (start * 2 + lsize <= len && start != 0) {
 			start <<= 1;
@@ -70,12 +70,12 @@ static int depack_unic(FILE *in, FILE *out)
 		write16b(out, lsize);
 	}
 
-	npat = read8(in);
+	npat = hio_read8(in);
 	write8(out, npat);			/* number of pattern */
 	write8(out, 0x7f);			/* noisetracker byte */
-	read8(in);
+	hio_read8(in);
 
-	fread(tmp, 128, 1, in);			/* pat table */
+	hio_read(tmp, 128, 1, in);			/* pat table */
 	fwrite(tmp, 128, 1, out);
 
 	/* get highest pattern number */
@@ -88,18 +88,18 @@ static int depack_unic(FILE *in, FILE *out)
 	write32b(out, PW_MOD_MAGIC);
 
 	/* verify UNIC ID */
-	fseek(in, 1080, SEEK_SET);
-	id = read32b(in);
+	hio_seek(in, 1080, SEEK_SET);
+	id = hio_read32b(in);
 
 	if (id && id != MAGIC_M_K_ && id != MAGIC_UNIC)
-		fseek(in, -4, SEEK_CUR);
+		hio_seek(in, -4, SEEK_CUR);
 
 	/* pattern data */
 	for (i = 0; i < max; i++) {
 		for (j = 0; j < 256; j++) {
-			c1 = read8(in);
-			c2 = read8(in);
-			c3 = read8(in);
+			c1 = hio_read8(in);
+			c2 = hio_read8(in);
+			c3 = hio_read8(in);
 
 			ins = ((c1 >> 2) & 0x10) | ((c2 >> 4) & 0x0f);
 			note = c1 & 0x3f;
