@@ -107,7 +107,16 @@ static int rad_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		return -1;
 
 	while ((b = hio_read8(f)) != 0) {
-		hio_read(sid, 1, 11, f);
+
+		/* Sanity check */
+		if (b > mod->ins || mod->xxs[b - 1].data != NULL) {
+			return -1;
+		}
+
+		if (hio_read(sid, 1, 11, f) != 11) {
+			return -1;
+		}
+
 		if (load_sample(m, f, SAMPLE_FLAG_ADLIB | SAMPLE_FLAG_HSC,
 					&mod->xxs[b - 1], (char *)sid) < 0) {
 			return -1;
@@ -163,13 +172,15 @@ static int rad_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 			if ((r & 0x7f) >= 64) {
 				D_(D_CRIT "** Whoops! row = %d\n", r);
+				return -1;
 			}
 
 			do {
 				c = hio_read8(f);	/* Channel number */
 
-				if ((c & 0x7f) >= mod->chn) {
-					D_(D_CRIT "** Whoops! channel = %d\n", c);
+				/* Sanity check */
+				if ((c & 0x7f) >= mod->chn || (r & 0x7f) >= 64) {
+					return -1;
 				}
 
 				event = &EVENT(i, c & 0x7f, r & 0x7f);
