@@ -36,7 +36,6 @@
 #include "loader.h"
 #include "xm.h"
 
-#define MAX_SAMP 1024
 
 static int xm_test (HIO_HANDLE *, char *, const int);
 static int xm_load (struct module_data *, HIO_HANDLE *, const int);
@@ -315,7 +314,7 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
     D_(D_INFO "Instruments: %d", mod->ins);
 
     /* ESTIMATED value! We don't know the actual value at this point */
-    mod->smp = MAX_SAMP;
+    mod->smp = MAX_SAMPLES;
 
     if (instrument_init(mod) < 0)
 	return -1;
@@ -340,6 +339,7 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 
 	/* Sanity check */
 	if (xih.samples > 0x10 || (xih.samples > 0 && xih.sh_size > 0x100)) {
+		D_(D_CRIT "Sanity check: %d %d", xih.samples, xih.sh_size);
 		break;
 	}
 
@@ -557,18 +557,34 @@ static int xm_load(struct module_data *m, HIO_HANDLE *f, const int start)
     xfh.bpm = hio_read16l(f);		/* Default BPM */
 
     /* Sanity checks */
-    if (xfh.songlen > 255 || xfh.patterns > 255 || xfh.instruments > 255)
+    if (xfh.songlen > 256 || xfh.patterns > 256 || xfh.instruments > 255) {
+	D_(D_CRIT "Sanity check: %d %d %d", xfh.songlen, xfh.patterns,
+							xfh.instruments);
 	return -1;
+    }
 
-    if (xfh.restart > 255 || xfh.channels > XMP_MAX_CHANNELS)
+    if (xfh.restart > 255 || xfh.channels > XMP_MAX_CHANNELS) {
+	D_(D_CRIT "Sanity check: %d %d", xfh.restart, xfh.channels);
         return -1;
+    }
 
-    if (xfh.tempo <= 0 || xfh.tempo >= 32 || xfh.bpm < 32 || xfh.bpm > 255)
-	return -1;
+    if (xfh.tempo < 0) {
+	D_(D_CRIT "Sanity check: %d", xfh.tempo);
+        return -1;
+    }
+
+    if (xfh.tempo >= 32 || xfh.bpm < 32 || xfh.bpm > 255) {
+        if (memcmp("MED2XM", xfh.tracker, 6)) {
+	    D_(D_CRIT "Sanity check: %d %d", xfh.tempo, xfh.bpm);
+	    return -1;
+        }
+    }
 
     len = xfh.headersz - 0x14;
-    if (len < 0 || len > 256)
+    if (len < 0 || len > 256) {
+	D_(D_CRIT "Sanity check: %d", len);
 	return -1;
+    }
 
     /* Honor header size -- needed by BoobieSqueezer XMs */
     hio_read(&xfh.order, len, 1, f); /* Pattern order table */
