@@ -377,8 +377,13 @@ void mixer_softmixer(struct context_data *ctx)
 		vi->pos0 = vi->pos;
 
 		buf_pos = s->buf32;
-		vol_r = vi->vol * (0x80 - vi->pan);
-		vol_l = vi->vol * (0x80 + vi->pan);
+		if (vi->pan == PAN_SURROUND) {
+			vol_r = vi->vol * 0x80;
+			vol_l = -vi->vol * 0x80;
+		} else {
+			vol_r = vi->vol * (0x80 - vi->pan);
+			vol_l = vi->vol * (0x80 + vi->pan);
+		}
 
 #ifndef LIBXMP_CORE_PLAYER
 		if (vi->fidx & FLAG_SYNTH) {
@@ -398,10 +403,11 @@ void mixer_softmixer(struct context_data *ctx)
 			continue;
 		}
 
-		if (vi->smp < mod->smp)
+		if (vi->smp < mod->smp) {
 			xxs = &mod->xxs[vi->smp];
-		else
+		} else {
 			xxs = &ctx->smix.xxs[vi->smp - mod->smp];
+		}
 
 		lps = xxs->lps;
 		lpe = xxs->lpe;
@@ -411,6 +417,12 @@ void mixer_softmixer(struct context_data *ctx)
 		}
 
 		for (size = s->ticksize; size > 0; ) {
+			int split_noloop = 0;
+
+			if (p->xc_data[vi->chn].split) {
+				split_noloop = 1;
+			}
+
 			/* How many samples we can write before the loop break
 			 * or sample end... */
 			if (vi->pos >= vi->end) {
@@ -445,10 +457,11 @@ void mixer_softmixer(struct context_data *ctx)
 				}
 
 #ifndef LIBXMP_CORE_DISABLE_IT
-				/* "Beautiful Ones" apparently uses 0xfe as
-				 * 'no filter' :\ */
-				if (vi->filter.cutoff >= 0xfe)
+				/* See OpenMPT env-flt-max.it */
+				if (vi->filter.cutoff >= 0xfe &&
+                                    vi->filter.resonance == 0) {
 					mixer &= ~FLAG_FILTER;
+				}
 #endif
 
 				mix_fn = (*mixers)[mixer];
@@ -478,7 +491,7 @@ void mixer_softmixer(struct context_data *ctx)
 				continue;
 
 			/* First sample loop run */
-			if (~xxs->flg & XMP_SAMPLE_LOOP) {
+			if ((~xxs->flg & XMP_SAMPLE_LOOP) || split_noloop) {
 				anticlick(ctx, voc, 0, 0, buf_pos, size);
 				set_sample_end(ctx, voc, 1);
 				size = 0;
@@ -646,6 +659,13 @@ void mixer_setnote(struct context_data *ctx, int voc, int note)
 	struct player_data *p = &ctx->p;
 	struct mixer_voice *vi = &p->virt.voice_array[voc];
 
+	/* FIXME: Workaround for crash on notes that are too high
+	 *        see 6nations.it (+114 transposition on instrument 16)
+	 */
+	if (note > 149) {
+		note = 149;
+	}
+
 	vi->note = note;
 	vi->period = note_to_period_mix(note, 0);
 	vi->attack = SLOW_ATTACK;
