@@ -144,7 +144,7 @@ static void reset_channels(struct context_data *ctx)
 		memset(xc, 0, sizeof (struct channel_data));
 		xc->extra = extra;
 		reset_channel_extras(ctx, xc);
-		xc->ins = 0;
+		xc->ins = -1;
 		xc->old_ins = 1;	/* raw value */
 		xc->key = -1;
 		xc->volume = m->volbase;
@@ -153,7 +153,7 @@ static void reset_channels(struct context_data *ctx)
 	for (i = 0; i < p->virt.virt_channels; i++) {
 		xc = &p->xc_data[i];
 		memset(xc, 0, sizeof (struct channel_data));
-		xc->ins = 0;
+		xc->ins = -1;
 		xc->old_ins = 1;	/* raw value */
 		xc->key = -1;
 		xc->volume = m->volbase;
@@ -165,6 +165,18 @@ static void reset_channels(struct context_data *ctx)
 		xc->pan.val = mod->xxc[i].pan;
 		xc->mastervol = mod->xxc[i].vol;
 		xc->filter.cutoff = 0xff;
+		
+		/* Amiga split channel */
+		if (mod->xxc[i].flg & XMP_CHANNEL_SPLIT) {
+			xc->split = ((mod->xxc[i].flg & 0x30) >> 4) + 1;
+		} else {
+			xc->split = 0;
+		}
+
+		/* Surround channel */
+		if (mod->xxc[i].flg & XMP_CHANNEL_SURROUND) {
+			xc->pan.surround = 1;
+		}
 	}
 }
 
@@ -260,6 +272,9 @@ static inline void read_row(struct context_data *ctx, int pat, int row)
 				med_hold_hack(ctx, pat, chn, row);
 #endif
 			}
+		} else {
+			/* Reset flags. See SlideDelay.it */
+			p->xc_data[chn].flags = 0;
 		}
 	}
 }
@@ -340,6 +355,12 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 		}
 	}
 
+	if (!TEST_PER(VENV_PAUSE)) {
+		xc->v_idx = update_envelope(&instrument->aei, xc->v_idx,
+					DOENV_RELEASE, TEST(KEY_OFF),
+					m->read_event_type == READ_EVENT_IT);
+	}
+
 	vol_envelope = get_envelope(&instrument->aei, xc->v_idx, 64);
 	if (check_envelope_end(&instrument->aei, xc->v_idx)) {
 		if (vol_envelope == 0)
@@ -347,12 +368,6 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 		SET_NOTE(NOTE_ENV_END);
 	}
 
-	if (!TEST_PER(VENV_PAUSE)) {
-		xc->v_idx = update_envelope(&instrument->aei, xc->v_idx,
-			DOENV_RELEASE, m->read_event_type == READ_EVENT_IT);
-	}
-
-
 	/* If note ended in background channel, we can safely reset it */
 	if (TEST_NOTE(NOTE_END) && chn >= p->virt.num_tracks) {
 		virt_resetchannel(ctx, chn);
@@ -424,7 +439,21 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 
 	xc->info_finalvol = TEST_NOTE(NOTE_SAMPLE_END) ? 0 :finalvol;
 
-	virt_setvol(ctx, chn, finalvol);
+	/* Check Amiga split channel */
+	if (xc->split) {
+		int i;
+
+		/* Also set the volume of the paired channel */
+		for (i = 0; i < m->mod.chn; i++) {
+			if (i < chn && p->xc_data[i].split == xc->split) {
+				virt_setvol(ctx, i, finalvol);
+				virt_setvol(ctx, chn, finalvol);
+				break;
+			}
+		}
+	} else {
+		virt_setvol(ctx, chn, finalvol);
+	}
 }
 
 static void process_frequency(struct context_data *ctx, int chn, int act)
@@ -446,11 +475,12 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 
 	instrument = get_instrument(ctx, xc->ins);
 
-	frq_envelope = get_envelope(&instrument->fei, xc->f_idx, 0);
 	if (!TEST_PER(FENV_PAUSE)) {
 		xc->f_idx = update_envelope(&instrument->fei, xc->f_idx,
-			DOENV_RELEASE, m->read_event_type == READ_EVENT_IT);
+					DOENV_RELEASE, TEST(KEY_OFF),
+					m->read_event_type == READ_EVENT_IT);
 	}
+	frq_envelope = get_envelope(&instrument->fei, xc->f_idx, 0);
 
 #ifndef LIBXMP_CORE_PLAYER
 	/* Do note slide */
@@ -463,6 +493,8 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 			xc->noteslide.count = xc->noteslide.speed;
 		}
 		xc->noteslide.count--;
+
+		virt_setnote(ctx, chn, xc->note);
 	}
 #endif
 
@@ -496,8 +528,8 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	period += extras_get_period(ctx, xc);
 #endif
 
-	linear_bend = period_to_bend(period + vibrato, xc->note, xc->gliss,
-				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+	linear_bend = period_to_bend(period + vibrato, xc->note,
+			TEST(GLISSANDO), HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 
 	/* Envelope */
 
@@ -542,9 +574,10 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	}
 
 	if (xc->f_idx >= 0 && (instrument->fei.flg & XMP_ENVELOPE_FLT)) {
-		cutoff = xc->filter.cutoff * frq_envelope >> 8;
-		if (cutoff > 0xfd)
-			cutoff = 0xfd;
+		if (frq_envelope < 0xfe) {
+			xc->filter.envelope = frq_envelope;
+		}
+		cutoff = xc->filter.cutoff * xc->filter.envelope >> 8;
 	} else {
 		cutoff = xc->filter.cutoff;
 	}
@@ -579,11 +612,12 @@ static void process_pan(struct context_data *ctx, int chn, int act)
 
 	instrument = get_instrument(ctx, xc->ins);
 
-	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32);
 	if (!TEST_PER(PENV_PAUSE)) {
 		xc->p_idx = update_envelope(&instrument->pei, xc->p_idx,
-			DOENV_RELEASE, m->read_event_type == READ_EVENT_IT);
+					DOENV_RELEASE, TEST(KEY_OFF),
+					m->read_event_type == READ_EVENT_IT);
 	}
+	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32);
 
 	if (TEST(PANBRELLO)) {
 		panbrello = get_lfo(ctx, &xc->panbrello.lfo, 512, 0);
@@ -593,7 +627,7 @@ static void process_pan(struct context_data *ctx, int chn, int act)
 	finalpan = xc->pan.val + panbrello + (pan_envelope - 32) *
 				(128 - abs(xc->pan.val - 128)) / 32;
 
-	if (s->format & XMP_FORMAT_MONO) {
+	if (s->format & XMP_FORMAT_MONO || xc->pan.surround) {
 		finalpan = 0;
 	} else {
 		finalpan = (finalpan - 0x80) * s->mix / 100;
@@ -603,7 +637,11 @@ static void process_pan(struct context_data *ctx, int chn, int act)
 
 	xc->info_finalpan = finalpan + 0x80;
 
-	virt_setpan(ctx, chn, finalpan);
+	if (xc->pan.surround) {
+		virt_setpan(ctx, chn, PAN_SURROUND);
+	} else {
+		virt_setpan(ctx, chn, finalpan);
+	}
 }
 
 static void update_volume(struct context_data *ctx, int chn)
@@ -650,6 +688,11 @@ static void update_volume(struct context_data *ctx, int chn)
 		if (TEST(FINE_VOLS))
 			xc->volume += xc->vol.fslide;
 
+#ifndef LIBXMP_CORE_DISABLE_IT
+		if (TEST(FINE_VOLS_2))
+			xc->volume += xc->vol.fslide2;
+#endif
+
 		if (TEST(TRK_FVSLIDE))
 			xc->mastervol += xc->trackvol.fslide;
 
@@ -806,6 +849,8 @@ static void play_channel(struct context_data *ctx, int chn)
 			if (xc->retrig.type < 0x10) {
 				/* don't retrig on cut */
 				virt_voicepos(ctx, chn, 0);
+			} else {
+				SET_NOTE(NOTE_END);
 			}
 			xc->volume += rval[xc->retrig.type].s;
 			xc->volume *= rval[xc->retrig.type].m;
@@ -834,6 +879,10 @@ static void play_channel(struct context_data *ctx, int chn)
 	}
 #endif
 
+	if (TEST_NOTE(NOTE_SUSEXIT)) {
+		SET_NOTE(NOTE_RELEASE);
+	}
+
 	xc->info_position = virt_getvoicepos(ctx, chn);
 }
 
@@ -1194,12 +1243,15 @@ int xmp_play_frame(xmp_context opaque)
 		}
 	}
 
-	/* check new row */
+	for (i = 0; i < mod->chn; i++) {
+		struct channel_data *xc = &p->xc_data[i];
+		RESET(KEY_OFF);
+	}
+
+	/* check new r w */
 
 	if (p->frame == 0) {			/* first frame in row */
-
 		check_end_of_module(ctx);
-
 		read_row(ctx, mod->xxo[p->ord], p->row);
 
 #ifndef LIBXMP_CORE_PLAYER
