@@ -54,6 +54,9 @@ struct retrig_control {
 #define VENV_PAUSE	(1 << 22)
 #define PENV_PAUSE	(1 << 23)
 #define FENV_PAUSE	(1 << 24)
+#define FINE_VOLS_2	(1 << 25)
+#define GLISSANDO	(1 << 26)
+#define KEY_OFF		(1 << 27)	/* for IT release on envloop end */
 
 #define NOTE_FADEOUT	(1 << 0)
 #define NOTE_RELEASE	(1 << 1)
@@ -62,6 +65,7 @@ struct retrig_control {
 #define NOTE_ENV_END	(1 << 4)
 #define NOTE_SAMPLE_END	(1 << 5)
 #define NOTE_SET	(1 << 6)	/* for IT portamento after keyoff */
+#define NOTE_SUSEXIT	(1 << 7)	/* for delayed note release */
 
 #define IS_VALID_INSTRUMENT(x) ((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0)
 #define IS_VALID_INSTRUMENT_OR_SFX(x) (((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0) || (smix->ins > 0 && (uint32)(x) < mod->ins + smix->ins))
@@ -88,7 +92,7 @@ struct channel_data {
 	int keyoff;		/* Key off counter */
 	int fadeout;		/* Current fadeout (release) value */
 	int ins_fade;		/* Instrument fadeout value */
-	int gliss;		/* Glissando active */
+	int split;		/* Split channel */
 	int volume;		/* Current volume */
 	int gvl;		/* Global volume for instrument for IT */
 
@@ -96,6 +100,8 @@ struct channel_data {
 	int p_idx;		/* Pan envelope index */
 	int f_idx;		/* Freq envelope index */
 
+	int key_porta;		/* Key number for portamento target
+				 * -- needed to handle IT portamento xpo */
 	struct {
 		struct lfo lfo;
 		int memory;
@@ -148,6 +154,10 @@ struct channel_data {
 		int fslide;	/* Fine volume slide value */
 		int slide2;	/* Volume slide value */
 		int memory;	/* Volume slide effect memory */
+#ifndef LIBXMP_CORE_DISABLE_IT
+		int fslide2;
+		int memory2;	/* Volume slide effect memory */
+#endif
 	} vol;
 
 	struct {
@@ -190,6 +200,7 @@ struct channel_data {
 		int slide;	/* Pan slide value */
 		int fslide;	/* Pan fine slide value */
 		int memory;	/* Pan slide effect memory */
+		int surround;	/* Surround channel flag */
 	} pan;	
 
 	struct {
@@ -206,6 +217,7 @@ struct channel_data {
 	struct {
 		int cutoff;	/* IT filter cutoff frequency */
 		int resonance;	/* IT filter resonance */
+		int envelope;	/* IT filter envelope */
 	} filter;
 
 #endif
