@@ -21,6 +21,7 @@
  */
 
 #include <string.h>
+#include <stdlib.h>
 #include "common.h"
 #include "player.h"
 #include "effects.h"
@@ -41,6 +42,16 @@ static inline void copy_channel(struct player_data *p, int to, int from)
 	}
 }
 
+static inline int has_note_event(struct xmp_event *e)
+{
+	return (e->note && e->note <= XMP_MAX_KEYS);
+}
+
+static inline int is_valid_note(int note)
+{
+	return (note >= 0 && note < XMP_MAX_KEYS);
+}
+
 static struct xmp_subinstrument *get_subinstrument(struct context_data *ctx,
 						   int ins, int key)
 {
@@ -50,7 +61,7 @@ static struct xmp_subinstrument *get_subinstrument(struct context_data *ctx,
 
 	if (IS_VALID_INSTRUMENT(ins)) {
 		instrument = &mod->xxi[ins];
-		if (key >= 0 && key < XMP_MAX_KEYS) {
+		if (is_valid_note(key)) {
 			int mapped = instrument->map[key].ins;
 			if (mapped != 0xff && mapped >= 0 && mapped < instrument->nsm)
 			  	return &instrument->sub[mapped];
@@ -70,9 +81,9 @@ static void reset_envelopes(struct context_data *ctx, struct channel_data *xc)
 
  	RESET_NOTE(NOTE_ENV_END);
 
-	xc->v_idx = 0;
-	xc->p_idx = 0;
-	xc->f_idx = 0;
+	xc->v_idx = -1;
+	xc->p_idx = -1;
+	xc->f_idx = -1;
 }
 
 #ifndef LIBXMP_CORE_DISABLE_IT
@@ -93,18 +104,13 @@ static void reset_envelopes_carry(struct context_data *ctx,
 
 	/* Reset envelope positions */
 	if (~xxi->aei.flg & XMP_ENVELOPE_CARRY) {
-		xc->v_idx = 0;
+		xc->v_idx = -1;
 	}
 	if (~xxi->pei.flg & XMP_ENVELOPE_CARRY) {
-		xc->p_idx = 0;
+		xc->p_idx = -1;
 	}
-
-	if (xxi->fei.flg & XMP_ENVELOPE_CARRY) {
-		if (check_envelope_end(&xxi->fei, xc->f_idx)) {
-			xc->f_idx = -1;
-		}
-	} else {
-		xc->f_idx = 0;
+	if (~xxi->fei.flg & XMP_ENVELOPE_CARRY) {
+		xc->f_idx = -1;
 	}
 }
 
@@ -115,14 +121,17 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 				struct channel_data *xc, int is_toneporta)
 {
 	if (sub != NULL && note >= 0) {
+		struct xmp_instrument *xxi = &ctx->m.mod.xxi[xc->ins];
+
 		xc->finetune = sub->fin;
 		xc->gvl = sub->gvl;
 
 		if (sub->ifc & 0x80) {
 			xc->filter.cutoff = (sub->ifc - 0x80) * 2;
-		} /*else {
+		} else if (~xxi->fei.flg & XMP_ENVELOPE_FLT) {
 			xc->filter.cutoff = 0xff;
-		} */
+		}
+		xc->filter.envelope = 0x100;
 
 		if (sub->ifr & 0x80) {
 			xc->filter.resonance = (sub->ifr - 0x80) * 2;
@@ -232,7 +241,7 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 		if (IS_VALID_INSTRUMENT(ins)) {
 			if (is_toneporta) {
 				/* Get new instrument volume */
-				sub = get_subinstrument(ctx, ins, e->note);
+				sub = get_subinstrument(ctx, ins, e->note - 1);
 				if (sub != NULL) {
 					xc->volume = sub->vol;
 					use_ins_vol = 0;
@@ -301,9 +310,6 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 	process_fx(ctx, xc, chn, e, 0);
 	set_period(ctx, note, sub, xc, is_toneporta);
 
-	if (TEST(NEW_VOL))
-		use_ins_vol = 0;
-
 	if (sub == NULL) {
 		return 0;
 	}
@@ -320,14 +326,21 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 		RESET(OFFSET);
 	}
 
-	if (use_ins_vol) {
+	if (use_ins_vol && !TEST(NEW_VOL)) {
 		xc->volume = sub->vol;
-		SET(NEW_VOL);
 	}
 
 	return 0;
 }
 
+static int sustain_check(struct xmp_envelope *env, int idx)
+{
+	return (env &&
+		(env->flg & XMP_ENVELOPE_ON) &&
+		(~env->flg & XMP_ENVELOPE_LOOP) &&
+		idx == env->data[env->sus << 1]);
+}
+
 static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn)
 {
 	struct player_data *p = &ctx->p;
@@ -419,7 +432,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		use_ins_vol = 1;
 		xc->fadeout = 0x10000;
 		xc->per_flags = 0;
-		RESET_NOTE(NOTE_RELEASE);
+		RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT);
 		if (!k00) {
 			RESET_NOTE(NOTE_FADEOUT);
 		}
@@ -474,6 +487,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			int env_on = 0;
 			int vol_set = ev.vol != 0 || ev.fxt == FX_VOLSET;
 			int delay_fx = ev.fxt == FX_EXTENDED && ev.fxp == 0xd0;
+			struct xmp_envelope *env = NULL;
 
 			/* OpenMPT NoteOffVolume.xm:
 			 * "If an instrument has no volume envelope, a note-off
@@ -484,13 +498,23 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			 * ...and unless we have a keyoff+delay without setting
 			 * an instrument. See OffDelay.xm.
 			 */
-			if (IS_VALID_INSTRUMENT(xc->ins) &&
-			    (mod->xxi[xc->ins].aei.flg & XMP_ENVELOPE_ON)) {
-				env_on = 1;
+			if (IS_VALID_INSTRUMENT(xc->ins)) {
+				env = &mod->xxi[xc->ins].aei;
+				if (env->flg & XMP_ENVELOPE_ON) {
+					env_on = 1;
+				}
 			}
 			
 			if (env_on || (!vol_set && (!ev.ins || !delay_fx))) {
-				SET_NOTE(NOTE_RELEASE);
+				if (sustain_check(env, xc->v_idx)) {
+					/* See OpenMPT EnvOff.xm. In certain
+					 * cases a release event is effective
+					 * only in the next frame
+					 */
+					SET_NOTE(NOTE_SUSEXIT);
+				} else {
+					SET_NOTE(NOTE_RELEASE);
+				}
 				use_ins_vol = 0;
 			} else {
 				SET_NOTE(NOTE_FADEOUT);
@@ -501,7 +525,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			    (ev.fxp >> 4) == EX_DELAY) {
 				/* See OpenMPT OffDelay.xm test case */
 				if ((ev.fxp & 0xf) != 0) {
-					RESET_NOTE(NOTE_RELEASE);
+					RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT);
 				}
 			}
 		} else if (key == XMP_KEY_FADE) {
@@ -536,7 +560,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	 *  memory). The instrument number next it, however, is not affected
 	 *  and remains in the memory."
 	 */
-	if (key > 0 && (uint32)key <= XMP_MAX_KEYS) {
+	if (is_valid_note(key - 1)) {
 		int k = key - 1;
 		sub = get_subinstrument(ctx, xc->ins, k);
 		if (!new_invalid_ins && sub != NULL) {
@@ -549,7 +573,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		}
 	}
 
-	if (key > 0 && (uint32)key <= XMP_MAX_KEYS) {
+	if (is_valid_note(key - 1)) {
 		xc->key = --key;
 		xc->fadeout = 0x10000;
 		RESET_NOTE(NOTE_END);
@@ -611,9 +635,6 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	process_fx(ctx, xc, chn, &ev, 0);
 	set_period_ft2(ctx, note, sub, xc, is_toneporta);
 
-	if (TEST(NEW_VOL))
-		use_ins_vol = 0;
-
 	if (sub == NULL) {
 		return 0;
 	}
@@ -639,9 +660,8 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		}
 	}
 
-	if (use_ins_vol) {
+	if (use_ins_vol && !TEST(NEW_VOL)) {
 		xc->volume = sub->vol;
-		SET(NEW_VOL);
 	}
 
 	return 0;
@@ -693,7 +713,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 					xc->ins_fade = mod->xxi[ins].rls;
 				} else {
 					/* Get new instrument volume */
-					sub = get_subinstrument(ctx, ins, e->note);
+					sub = get_subinstrument(ctx, ins, e->note - 1);
 					if (sub != NULL) {
 						xc->volume = sub->vol;
 						use_ins_vol = 0;
@@ -770,9 +790,6 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 	process_fx(ctx, xc, chn, e, 0);
 	set_period(ctx, note, sub, xc, is_toneporta);
 
-	if (TEST(NEW_VOL))
-		use_ins_vol = 0;
-
 	if (sub == NULL) {
 		return 0;
 	}
@@ -782,9 +799,8 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 		virt_voicepos(ctx, chn, xc->offset.val);
 	}
 
-	if (use_ins_vol) {
+	if (use_ins_vol && !TEST(NEW_VOL)) {
 		xc->volume = sub->vol;
-		SET(NEW_VOL);
 	}
 
 	/* ST3: check QUIRK_ST3GVOL only in ST3 event reader */
@@ -797,6 +813,35 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 
 #ifndef LIBXMP_CORE_DISABLE_IT
 
+static int check_fadeout(struct context_data *ctx, struct channel_data *xc, int ins)
+{
+	struct xmp_instrument *xxi = get_instrument(ctx, ins);
+
+	if (xxi == NULL) {
+		return 1;
+	}
+
+	return (~xxi->aei.flg & XMP_ENVELOPE_ON ||
+		~xxi->aei.flg & XMP_ENVELOPE_CARRY ||
+		xc->ins_fade == 0 ||
+		xc->fadeout <= xc->ins_fade);
+}
+
+static int check_invalid_sample(struct context_data *ctx, int ins, int key)
+{
+	struct module_data *m = &ctx->m;
+	struct xmp_module *mod = &m->mod;
+
+	if (ins < mod->ins) {
+		int smp = mod->xxi[ins].map[key].ins;
+		if (smp == 0xff || smp >= mod->smp) {
+			return 1;
+		};
+	}
+
+	return 0;
+}
+
 static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 {
 	struct player_data *p = &ctx->p;
@@ -811,6 +856,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	int candidate_ins;
 	int reset_env;
 	int use_ins_vol;
+	int sample_mode;
 	struct xmp_event ev;
 
 	memcpy(&ev, e, sizeof (struct xmp_event));
@@ -823,6 +869,14 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		xc->delayed_ins = 0;
 	}
 
+	/* Keyoff + instrument retrigs current instrument in old fx mode */
+	if (HAS_QUIRK(QUIRK_ITOLDFX)) {
+		if (ev.note == XMP_KEY_OFF && IS_VALID_INSTRUMENT(ev.ins -1)) {
+			ev.note = xc->key + 1;
+			ev.ins = xc->ins + 1;
+		}
+	}
+
 	xc->flags = 0;
 	note = -1;
 	key = ev.note;
@@ -833,25 +887,24 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	reset_env = 0;
 	use_ins_vol = 0;
 	candidate_ins = xc->ins;
+	sample_mode = !HAS_QUIRK(QUIRK_VIRTUAL);
 
 	/* Notes with unmapped instruments are ignored */
 	if (ev.ins) {
-		if (ev.ins <= mod->ins && ev.note && ev.note <= XMP_MAX_KEYS) {
+		if (ev.ins <= mod->ins && has_note_event(&ev)) {
 			int ins = ev.ins - 1;
-			int key = ev.note - 1;
-
-			if (ins < mod->ins) {
-				int smp = mod->xxi[ins].map[key].ins;
-				if (smp == 0xff || smp >= mod->smp) {
-					candidate_ins = ins;
-					memset(&ev, 0, sizeof (ev));
-				};
+			if (check_invalid_sample(ctx, ins, ev.note - 1)) {
+				candidate_ins = ins;
+				memset(&ev, 0, sizeof (ev));
 			}
 		}
 	} else {
-		if (ev.note && ev.note <= XMP_MAX_KEYS) {
-			if (!IS_VALID_INSTRUMENT(xc->old_ins - 1)) {
+		if (has_note_event(&ev)) {
+			int ins = xc->old_ins - 1;
+			if (!IS_VALID_INSTRUMENT(ins)) {
 				new_invalid_ins = 1;
+			} else if (check_invalid_sample(ctx, ins, ev.note - 1)) {
+				memset(&ev, 0, sizeof (ev));
 			}
 		}
 	}
@@ -864,10 +917,6 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		is_release = 1;
 	}
 
-	if (!HAS_QUIRK(QUIRK_VIRTUAL) && virt_mapchannel(ctx, chn) < 0) {
-		is_toneporta = 0;
-	}
-
 	if (xc->period <= 0 || TEST_NOTE(NOTE_END)) {
 		is_toneporta = 0;
 	}
@@ -921,6 +970,11 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			/* valid ins */
 
 			if (!key) {
+				/* Retrig in new ins in sample mode */
+				if (sample_mode && TEST_NOTE(NOTE_END)) {
+					virt_voicepos(ctx, chn, 0);
+				}
+
 				/* IT: Reset note for every new != ins */
 				if (xc->ins == ins) {
 					SET(NEW_INS);
@@ -931,6 +985,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 				RESET_NOTE(NOTE_SET);
 			}
+
 			if (xc->ins != ins && (!is_toneporta || !HAS_QUIRK(QUIRK_PRENV))) {
 				struct xmp_subinstrument *s1, *s2;
 				s1 = get_subinstrument(ctx, xc->ins, xc->key);
@@ -951,6 +1006,11 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 				}
 			}
 		} else {
+			/* In sample mode invalid ins cut previous ins */
+			if (sample_mode) {
+				xc->volume = 0;
+			}
+
 			/* Ignore invalid instruments */
 			new_invalid_ins = 1;
 			xc->flags = 0;
@@ -973,7 +1033,16 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			xc->period = 0;
 			virt_resetchannel(ctx, chn);
 		} else if (key == XMP_KEY_OFF) {
-			SET_NOTE(NOTE_RELEASE);
+			struct xmp_envelope *env = NULL;
+			if (IS_VALID_INSTRUMENT(xc->ins)) {
+				env = &mod->xxi[xc->ins].aei;
+			}
+			if (sustain_check(env, xc->v_idx)) {
+				SET_NOTE(NOTE_SUSEXIT);
+			} else {
+				SET_NOTE(NOTE_RELEASE);
+			}
+			SET(KEY_OFF);
 			reset_env = 0;
 			use_ins_vol = 0;
 		} else {
@@ -986,15 +1055,18 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			 	 */
 				if (not_same_ins || TEST_NOTE(NOTE_END)) {
 					SET(NEW_INS);
-					RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
+					RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
 				} else {
+					if (is_valid_note(key - 1)) {
+						xc->key_porta = key - 1;
+					}
 					key = 0;
 				}
 			}
 		}
 	}
 
-	if ((uint32)key <= XMP_MAX_KEYS && key > 0 && !new_invalid_ins) {
+	if (is_valid_note(key - 1) && !new_invalid_ins) {
 		xc->key = --key;
 		RESET_NOTE(NOTE_END);
 
@@ -1037,14 +1109,6 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		}
 	}
 
-	/* Reset in case of new instrument and the previous envelope has
-	 * finished (OpenMPT test EnvReset.it). This must take place after
-	 * channel copies in case of NNA (see test/test.it)
-	 */
-	if (ev.ins && TEST_NOTE(NOTE_ENV_END)) {
-		reset_envelopes(ctx, xc);
-	}
-
 	if (IS_VALID_INSTRUMENT(candidate_ins)) {
 		if (xc->ins != candidate_ins) {
 			/* Reset envelopes if instrument changes */
@@ -1054,19 +1118,43 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		xc->ins_fade = mod->xxi[candidate_ins].rls;
 	}
 
+	/* Reset in case of new instrument and the previous envelope has
+	 * finished (OpenMPT test EnvReset.it). This must take place after
+	 * channel copies in case of NNA (see test/test.it)
+	 * Also if we have envelope in carry mode, check fadeout
+	 */
+	if (ev.ins && TEST_NOTE(NOTE_ENV_END)) {
+		if (check_fadeout(ctx, xc, candidate_ins)) {
+			reset_envelopes(ctx, xc);
+		} else {
+			reset_env = 0;
+		}
+	}
+
 	sub = get_subinstrument(ctx, xc->ins, xc->key);
 
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
 	if (sub != NULL) {
 		if (note >= 0) {
-			if (sub->pan >= 0)
+			int pan_swing = (sub->rvv & 0xff00) >> 8;
+			CLAMP(pan_swing, 0, 64);
+
+			if (sub->pan >= 0) {
 				xc->pan.val = sub->pan;
+				xc->pan.surround = 0;
+			}
+
 			if (TEST_NOTE(NOTE_CUT)) {
 				reset_envelopes(ctx, xc);
 			} else {
 				reset_envelopes_carry(ctx, xc);
 			}
 			RESET_NOTE(NOTE_CUT);
+
+			if (pan_swing) {
+				xc->pan.val = (xc->pan.val * (64 - pan_swing) +
+					(rand() % 256) * pan_swing) >> 6;
+			}
 		} else if (ev.ins) {
 			if (sub->pan >= 0)
 				xc->pan.val = sub->pan;
@@ -1074,25 +1162,21 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	}
 	
 	/* Process new volume */
-	if (ev.vol) {
+	if (ev.vol && (!TEST_NOTE(NOTE_CUT) || ev.ins != 0)) {
 		xc->volume = ev.vol - 1;
 		SET(NEW_VOL);
 	}
 
 	/* IT: always reset sample offset */
-	xc->offset.val = 0;
+	xc->offset.val &= ~0xffff;
 
 	/* According to Storlek test 25, Impulse Tracker handles the volume
-	 * column effects last.
+	 * column effects after the standard effects.
 	 */
 	process_fx(ctx, xc, chn, &ev, 0);
 	process_fx(ctx, xc, chn, &ev, 1);
 	set_period(ctx, note, sub, xc, is_toneporta);
 
-	if (TEST(NEW_VOL)) {
-		use_ins_vol = 0;
-	}
-
 	if (sub == NULL) {
 		return 0;
 	}
@@ -1104,15 +1188,22 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 	if (reset_env) {
 		if (ev.note) {
-			RESET_NOTE(NOTE_RELEASE | NOTE_FADEOUT);
+			RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
 		}
 		/* Set after copying to new virtual channel (see ambio.it) */
 		xc->fadeout = 0x10000;
 	}
 
-	if (use_ins_vol) {
+	if (use_ins_vol && !TEST(NEW_VOL)) {
+		int vol_swing = sub->rvv & 0xff;
+		CLAMP(vol_swing, 0, 100);
+
 		xc->volume = sub->vol;
-		SET(NEW_VOL);
+
+		if (vol_swing) {
+			xc->volume = (xc->volume * (100 - vol_swing) + 
+				(rand() % (xc->volume + 1)) * vol_swing) / 100;
+		}
 	}
 
 	return 0;
@@ -1157,7 +1248,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 		if (IS_VALID_INSTRUMENT(ins)) {
 			if (is_toneporta) {
 				/* Get new instrument volume */
-				sub = get_subinstrument(ctx, ins, e->note);
+				sub = get_subinstrument(ctx, ins, e->note - 1);
 				if (sub != NULL) {
 					xc->volume = sub->vol;
 					use_ins_vol = 0;
@@ -1256,10 +1347,6 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 	process_fx(ctx, xc, chn, e, 0);
 	set_period(ctx, note, sub, xc, is_toneporta);
 
-	if (TEST(NEW_VOL)) {
-		use_ins_vol = 0;
-	}
-
 	if (sub == NULL) {
 		return 0;
 	}
@@ -1269,9 +1356,8 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 		virt_voicepos(ctx, chn, xc->offset.val);
 	}
 
-	if (use_ins_vol) {
+	if (use_ins_vol && !TEST(NEW_VOL)) {
 		xc->volume = sub->vol;
-		SET(NEW_VOL);
 	}
 
 	return 0;
@@ -1354,7 +1440,6 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	}
 
 	xc->volume = e->vol - 1;
-	SET(NEW_VOL);
 
 	xc->note = note;
 	virt_voicepos(ctx, chn, xc->offset.val);
@@ -1371,6 +1456,10 @@ int read_event(struct context_data *ctx, struct xmp_event *e, int chn)
 	if (e->ins != 0)
 		xc->old_ins = e->ins;
 
+	if (TEST_NOTE(NOTE_SAMPLE_END)) {
+		SET_NOTE(NOTE_END);
+	}
+
 	if (chn >= m->mod.chn) {
 		return read_event_smix(ctx, e, chn);
 	} else switch (m->read_event_type) {
