@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -58,9 +58,9 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
     int gvl, bpm, speed, base_time, chn;
     int frame_count;
     double time, start_time;
-    int loop_chn, loop_count;
+    int loop_chn, loop_num, inside_loop;
     int pdelay = 0;
-    int loop_stack[XMP_MAX_CHANNELS];
+    int loop_count[XMP_MAX_CHANNELS];
     int loop_row[XMP_MAX_CHANNELS];
     struct xmp_event* event;
     int i, pat;
@@ -78,9 +78,11 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 			mod->xxp[pat]->rows ? mod->xxp[pat]->rows : 1);
     }
 
-    memset(loop_stack, 0, sizeof(int) * mod->chn);
-    memset(loop_row, 0, sizeof(int) * mod->chn);
-    loop_count = 0;
+    for (i = 0; i < mod->chn; i++) {
+	loop_count[i] = 0;
+	loop_row[i] = -1;
+    }
+    loop_num = 0;
     loop_chn = -1;
 
     gvl = mod->gvl;
@@ -113,11 +115,12 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
     gvol_memory = break_row = row_count = frame_count = 0;
     start_time = time = 0.0;
+    inside_loop = 0;
 
     while (42) {
 	if ((uint32)++ord >= mod->len) {
 	    if (mod->rst > mod->len || mod->xxo[mod->rst] >= mod->pat) {
-		ord = m->seq_data[chain].entry_point;
+		ord = ep;
 	    } else {
 		if (get_sequence(ctx, mod->rst) == chain) {
 	            ord = mod->rst;
@@ -150,9 +153,12 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 	    continue;
 	}
 
-	if (break_row < mod->xxp[pat]->rows && m->scan_cnt[ord][break_row]) {
-	    break;
-	} else if (break_row >= mod->xxp[pat]->rows) {
+	/* Loops can cross pattern boundaries, so check if we're not looping */
+	if (break_row < mod->xxp[pat]->rows) {
+	    if (m->scan_cnt[ord][break_row] && !inside_loop) {
+	        break;
+	    }
+	} else {
 	    break_row = 0;
 	}
 
@@ -197,7 +203,7 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 	    if (row_count > 512)	/* was 255, but Global trash goes to 318 */
 		goto end_module;
 
-	    if (!loop_count && m->scan_cnt[ord][row]) {
+	    if (!loop_num && m->scan_cnt[ord][row]) {
 		row_count--;
 		goto end_module;
 	    }
@@ -263,10 +269,12 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		    row_count = 0;
 		    if (parm) {
 			if (p->flags & XMP_FLAGS_VBLANK || parm < 0x20) {
-			    speed = parm;
+			    if (parm > 0) {
+			        speed = parm;
 #ifndef LIBXMP_CORE_PLAYER
-			    st26_speed = 0;
+			        st26_speed = 0;
 #endif
+                            }
 			} else {
 			    time += m->time_factor * frame_count * base_time / bpm;
 			    frame_count = 0;
@@ -285,19 +293,25 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
 		/* ST2.6 speed processing */
 
-		if (f1 == FX_ICE_SPEED) {
-		    st26_speed = (MSN(p1) << 8) | LSN(p1);
+		if (f1 == FX_ICE_SPEED && p1) {
+		    if (LSN(p1)) {
+		        st26_speed = (MSN(p1) << 8) | LSN(p1);
+		    } else {
+			st26_speed = MSN(p1);
+		    }
 		}
 #endif
 
 		if ((f1 == FX_S3M_SPEED && p1) || (f2 == FX_S3M_SPEED && p2)) {
 		    parm = (f1 == FX_S3M_SPEED) ? p1 : p2;
-		    frame_count += row_count * speed;
-		    row_count  = 0;
-		    speed = parm;
+		    if (parm > 0) {
+		        frame_count += row_count * speed;
+		        row_count  = 0;
+		        speed = parm;
 #ifndef LIBXMP_CORE_PLAYER
-		    st26_speed = 0;
+		        st26_speed = 0;
 #endif
+		    }
 		}
 
 		if ((f1 == FX_S3M_BPM && p1) || (f2 == FX_S3M_BPM && p2)) {
@@ -358,6 +372,9 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		    ord2 = (f1 == FX_JUMP) ? p1 : p2;
 		    break_row = 0;
 		    last_row = 0;
+
+		    /* prevent infinite loop, see OpenMPT PatLoop-Various.xm */
+		    inside_loop = 0;
 		}
 
 		if (f1 == FX_BREAK || f2 == FX_BREAK) {
@@ -377,25 +394,27 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		    if ((parm >> 4) == EX_PATTERN_LOOP) {
 			if (parm &= 0x0f) {
 			    /* Loop end */
-			    if (loop_stack[chn]) {
-				if (--loop_stack[chn])
+			    if (loop_count[chn]) {
+				if (--loop_count[chn]) {
+				    /* next iteraction */
 				    loop_chn = chn;
-				else {
-				    loop_count--;
+				} else {
+				    /* finish looping */
+				    loop_num--;
+				    inside_loop = 0;
 				    if (m->quirk & QUIRK_S3MLOOP)
 					loop_row[chn] = row;
 				}
 			    } else {
-				if (loop_row[chn] < row) {
-				    loop_stack[chn] = parm;
-				    loop_chn = chn;
-				    loop_count++;
-				}
+				loop_count[chn] = parm;
+				loop_chn = chn;
+				loop_num++;
 			    }
 			} else { 
 			    /* Loop start */
 			    loop_row[chn] = row - 1;
-			    if (HAS_QUIRK(QUIRK_FT2LOOP))
+			    inside_loop = 1;
+			    if (HAS_QUIRK(QUIRK_FT2BUGS))
 				break_row = row;
 			}
 		    }
